(require 'ede)

(defvar ede-vcs-root-directories '(".git" ".hg" ".bzr"))
(defvar ede-vcs-root-project-list nil)

(defclass ede-vcs-root-project (ede-project eieio-instance-tracker)
  ((tracking-symbol :initform 'ede-vcs-root-project-list)
   )
  "Project Type for detecting roots of VCS handled files."
  :method-invocation-order :depth-first)

(defmethod ede-project-root-directory ((this ede-vcs-root-project)
				       &optional file)
  "Return root directory for this project."
  (oref this :file))

(defun ede-vcs-root-dir (&optional dir)
  "Get root directory for current file if under version control."
  (when (not dir) (setq dir default-directory))
  (let ((dirs ede-vcs-root-directories)
	root current)
    (while (and (not root)
		(setq current (pop dirs)))
      (setq root (locate-dominating-file dir current)))
    root))

(defun ede-vcs-root-load (dir &optional rootproj)
  "Create object for vcs-root project."
  (ede-vcs-root-project
   dir
   :name dir
   :directory dir
   :targets nil
   :file (expand-file-name "." dir)))

(ede-add-project-autoload
 (ede-project-autoload "vcs-root"
		       :name "VCS ROOT"
		       :file 'vcs-root
		       :proj-file "."
		       :proj-root 'ede-vcs-root-dir
		       :class-sym 'ede-vcs-root-project
		       :load-type 'ede-vcs-root-load
		       :new-p nil
		       :safe-p t)
 'generic)

(defun my-get-project-root ()
  (interactive)
  (let ((proj ede-object-root-project))
    (if proj
	(message "Project root: %s"
		 (ede-project-root-directory proj))
      (message "No project for this buffer."))))

(provide 'vcs-root)
